#  -*- coding: utf-8 -*-
#
#  GSXML - A Content Import/Export Product
#
#  Uses a specific context to import/export marshalled data
#
#  Copyright (c) 2006-2007 InQuant GmbH
#
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 2 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program; if not, write to the Free Software
#  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
""" the content uses a context for data storage and is responsible
for marshalling
"""

__author__ = """Ramon Bartl <ramonski@ramonski.de>"""
__docformat__ = 'restructuredtext'
__revision__ = "$Revision: 39118 $"

import os
import logging
import csv
from StringIO import StringIO

from zope.interface import implements

from Products.ATContentTypes.interfaces import IBaseFolder
from Products.Marshall.registry import getComponent
from interfaces import *

LOGGER="gsxml.content"

def info(msg):
    logging.getLogger(LOGGER).info(msg)

def debug(msg):
    logging.getLogger(LOGGER).debug(msg)

def error(msg):
    logging.getLogger(LOGGER).error(msg)


class ATCTXMLMarshaller(object):
    """
    Marshalls ATContentType -> XML
    """
    implements( IXMLMarshaller )

    def __init__(self, context ):
        info( "ATCTXMLMarshaller.__init__: context=%s" % (context) )
        self.context = context
        self.marshaller = getComponent( "atxml" )

    def getxml(self):
        info( "ATCTXMLMarshaller.getxml" )
        content_type, length, data = self.marshaller.marshall( self.context )
        self._xml = data
        self._content_type = content_type
        self._length = length
        return self._xml

    xml = property(getxml, "Property of 'xml'")

class ATCTXMLDemarshaller(object):
    """
    Adapter which adapts any ATContentType to IXMLDemarshaller
    """
    implements( IXMLDemarshaller )

    def __init__( self, context ):
        info( "ATCTXMLDemarshaller.__init__: context=%s" % ( context ) )
        self.context = context
        self.marshaller = getComponent( "atxml" )

    def demarshall( self, instance, xml ):
        info( "ATCTXMLDemarshaller.demarshall: instance=%s" % ( instance ) )
        self.marshaller.demarshall( instance, xml )
        instance.reindexObject()

class XMLContentFSExporter(object):
    implements( IGSXMLFilesystemExporter )

    def __init__( self, context ):
        info( "XMLContentFSExporter.__init__: context=%s" % ( context ) )
        self.context = context

        self.folderish = IBaseFolder.isImplementedBy( context )
        info( "XMLContentFSExporter: context folderish: %s" % self.folderish )

    def write_manifest( self, export_context, subdir ):
        """
        Write a file which contains type,id lies which the importer
        will use to recreate these instances.
        """
        stream = StringIO()
        csv_writer = csv.writer(stream)

        for id, object, adapter in self.listExportableItems():
            """
            factory_namer = IContentFactoryName(object, None)
            if factory_namer is not None:
                factory_name = factory_namer()
            else:
                factory_name = object.portal_type
            """
            factory_name = object.portal_type
            csv_writer.writerow((id, factory_name))

        export_context.writeDataFile('.objects',
                                     text=stream.getvalue(),
                                     content_type='text/comma-separated-values',
                                     subdir=subdir,
                                    )

    # IGSXMLFilesystemExporter
    def export( self, export_context, subdir, root=False ):
        info( "XMLContentFSExporter.export: export_context=%s, subdir=%s, root=%s" % (
                export_context, subdir, root ) )

        if not IXMLExportableContent.providedBy( self.context ):
            error( "XMLContentFSExporter.export: cannot marshall %s" % self.context )
            return

        # Marshall our context
        xml = IXMLMarshaller( self.context ).xml
        fname = "%s.xml" % self.context.id
        path = os.path.join( subdir, fname )
        export_context.writeDataFile( path, xml, "text/xml" )

        if not self.folderish:
            return

        if not root:
            subdir = '%s/%s' % (subdir, self.context.getId())
            info( "XMLContentFSExporter: root! subdir=%s" % subdir )

        # Because we are a folder, export all our contents, too.
        # for this we need to fetch a list of objects which are
        # containd in the context, and export those for which
        # we can get and adapter to IGSXMLFilesystemExporter
        exportable = self.listExportableItems()

        for id, object, fse in exportable:
            if fse is None:
                continue
            info( "XMLContentFSExporter: about to export object %s using %s to %s" % 
                    ( object, fse, subdir ) )
            fse.export( export_context, subdir, False )

        self.write_manifest( export_context, subdir )


    def listExportableItems(self):
        """
        Return a list of tuples of exportable items:
          ( id, object, IGSXMLFilesystemExporter(obj) )

        i.e.
          [ ( "anobjectid", <ATDocument object ...>, <ATDocumentFilesystemExporter object ...> ), ( "nonexportable", <object>, None ), ... ]
        """
        if self.folderish:
            exportable = self.context.objectItems()
            exportable = [x + (IGSXMLFilesystemExporter(x[1], None),) for x in exportable]
            return exportable
        else:
            return ()

class XMLContentFSImporter(object):
    implements(IGSXMLFilesystemImporter)

    def __init__( self, context ):
        info( "XMLContentFSImporter.__init__: context=%s" % ( context ) )
        # this is the object which was newly created or is to be updated
        self.context = context

        # now lets look up adapter which provides IXMLDemarshaller for
        # this object:
        self.demarshaller = IXMLDemarshaller( self.context )

        self.folderish = IBaseFolder.isImplementedBy( context )
        info( "XMLContentFSImporter: context folderish: %s" % self.folderish )

    def debug(self):
        from zope import component
        for registration in component.getSiteManager().registrations():
            if isinstance(registration, component.site.AdapterRegistration):
                info( "XMLContentFSImporter.debug: %s" % registration )


    def read_manifest( self, import_context, subdir ):
        """
        Read the manifest file and return a list of tuples (objectid,ContentFactoryName)
        """
        objects = import_context.readDataFile('.objects', subdir)
        if objects is None:
            return []

        csvfile = csv.reader(StringIO(objects), dialect='excel' )
        return [ (id, typename) for id, typename in csvfile ]


    # IGSXMLFilesystemImporter
    def import_( self, import_context, subdir, root=False ):
        info( "XMLContentFSImporter.import: import_context=%s, subdir=%s, root=%s" % ( import_context, subdir, root ) )

        id = self.context.getId()
        filename = "%s.xml" % id
        xml = import_context.readDataFile( filename, subdir )
        if xml is None:
            return
        self.demarshaller.demarshall( self.context, xml )

        if not root:
            subdir = '%s/%s' % (subdir, self.context.getId())
            info( "XMLContentFSImporter: root! subdir=%s" % subdir )

        existing = self.context.objectIds()
        info( "XMLContentFSImporter.import: existing=%r" % ( list( existing ) ) ) 
        manifest = self.read_manifest( import_context, subdir )
        info( "XMLContentFSImporter.import: manifest=%r" % ( manifest ) ) 

        for object_id, type_name in manifest:
            info( "XMLContentFSImporter.import: object_id=%s, type_name=%s" % ( object_id, type_name ) )

            if object_id not in existing:
                #content_factory = queryAdapter( self.context, IContentFactory, name=type_name )
                #if content_factory is None:
                #    error( "XMLContentFSImporter: Could not create instance of type %s, id %s." % ( type_name, object_id ) )
                #    continue

                #content_factory(object_id)
                id = self.context.invokeFactory( type_name, object_id )
                if id is None:
                    error( "XMLContentFSImporter: Could not create instance of type %s, id %s." % ( type_name, object_id ) )
                    continue
                info( "XMLContentFSImporter: Created object type %s, id %s." % ( type_name, object_id ) )

            object = self.context._getOb(object_id)

            IGSXMLFilesystemImporter(object).import_(import_context, subdir)


# vim: set ft=python ts=4 sw=4 expandtab :
