#  -*- coding: utf-8 -*-
#
#  GSXML - A Content Import/Export Product
#
#  Handles Data Storage
#
#  Copyright (c) 2006-2007 InQuant GmbH
#
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 2 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program; if not, write to the Free Software
#  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
""" The context is responsible for data handling and storage
"""

__author__ = """Ramon Bartl <ramonski@ramonski.de>"""
__docformat__ = 'restructuredtext'
__revision__ = "$Revision: 39118 $"

import os
import logging
import datetime
from DateTime.DateTime import DateTime
from ConfigParser import SafeConfigParser
from StringIO import StringIO

import libgmail

from zope.interface import implements
from Products.GenericSetup.interfaces import IExportContext, IImportContext
from Products.GenericSetup.interfaces import IFilesystemImporter
from interfaces import *


SKIPPED_FILES = ('CVS', '.svn', '_svn', '_darcs')
SKIPPED_SUFFIXES = ('~',)

LOGGER="gsxml.context"

def info(msg):
    logging.getLogger(LOGGER).info(msg)

def debug(msg):
    logging.getLogger(LOGGER).debug(msg)

def error(msg):
    logging.getLogger(LOGGER).error(msg)


class GMessage(object):
    """
    class implementing sub-set of 'email.Message.Message' interface
    ('get_filename', 'get_content_type', 'get_payload'). This is to allow use
    of payloads from Message instances.
    """
    def __init__(self, filename, content, content_type):
        self.filename = filename
        self.content = content
        self.content_type = content_type

    def get_filename(self):
        return self.filename

    def get_content_type(self):
        return self.content_type

    def get_payload(self, i=None, decode=None):
        return self.content


class GmailContext(object):
    """
    a class that provide access to Google's Gmail web-mail service.
    """
    def __init__(self, user, password, subject, exportid):
        """
        initialize a gmail account object with user and password.
        """

        self.subject = subject
        self.exportid = exportid
        self.name = user
        self.pw = password
        self.ga = self.g_login( self.name, self.pw )

    def get_metadata(self, filename, **kwargs):
        """
        creates a metadata file which provides a structure similar to what you
        would find on Microsoft Windows INI files. The configuration file
        consists of sections, led by a "[section]" header and followed by
        "name: value" entries
        """
        scp = SafeConfigParser()
        scp.add_section("info")
        scp.set("info", "filename", filename)
        scp.set("info", "exportid", self.exportid)
        scp.set("info", "export_date", datetime.datetime.now().isoformat() )

        # additional given arguments go into the [extra] section
        scp.add_section("extra")
        for k,v in kwargs.iteritems():
            scp.set("extra", k, v )

        io = StringIO()
        scp.write(io)

        return io.getvalue()


    def get_subject(self):
        """
        specify the mail subject
        """
        return "%s %s" % ( self.subject, self.exportid )

    def get_body(self):
        """
        specify the mail body
        """
        return "Export from GSXML"

    def g_login(self, user, pw):
        """
        Login into GoogleMail Account
        'user' -- username for google mail account
        'pw' -- password for google mail account
        """

        ga = libgmail.GmailAccount(user, pw)
        info( "Please wait, logging in...." )

        try:
            ga.login()
        except libgmail.GmailLoginFailure:
            info( "Login failed. (Wrong username/password?)" )
            raise RuntimeError( "Login failed" )
        else:
            info( "Login successful." )
            return ga

    def g_msgquery(self, ga, query):
        """
        Search the Google Mail Account for 'query' string
        'ga' -- google mail account object
        'query' -- search string
        """
        messages = []
        result = ga.getMessagesByQuery( query )
        for th in result:
            for m in th:
                messages.append(m)
        return messages


    # Send a Message
    def g_send(self, filename, subject, content, content_type="text/xml" ):
        """
        Send GMail
        """

        # create a INI metadata attachment
        meta = GMessage( "metadata", self.get_metadata(filename), "text/csv" )
        # create the export xml attachment
        msg = GMessage( filename, content, content_type )

        gmsg = libgmail.GmailComposedMessage(self.to, self.get_subject(), self.get_body(), None, None, None, [meta, msg])

        if self.ga.sendMessage(gmsg):
            info( "Message sent '%s' successfully." % subject )
        else:
            info( "Could not send message." )


class GmailExportContext(GmailContext):
    """
    see IExportContext
    """
    implements( IExportContext )

    def __init__(self, name, password, to=None, subject="[GSXML]", msg=None, exportid=datetime.datetime.now().isoformat() ):
        GmailContext.__init__(self, name, password, subject, exportid )

        self.to = to or name
        self.msg = msg or "GSXML Export"

    # IExportContext
    def writeDataFile( self, filename, text, content_type, subdir=None ):
        info( "GmailExportContext.writeDataFile: filename=%s, subdir=%s, content_type=%s" % (
                filename, subdir, content_type ) )

        if subdir is not None:
            fn = os.path.join(  subdir, filename )
        else:
            fn = filename

        self.g_send( fn, self.subject, text, content_type )


class DictFSEnv(object):
    """
    stores data into a dictionary with a fs like manner
    """
    implements( IFSEnv )

    def __init__(self, brain):
        """
        initialize the storage with a dictionary
        """
        info( "DictFSEnv: __init__: %(brain)s" % locals() )
        self.brain = brain

    def lookup(self, path):
        dirs = path.split(os.sep)
        current = self.brain
        for dir in dirs:
            current = current[dir]
        return current

    def write(self, path, content):
        info( "DictFSEnv: write: %(path)s" % locals() )
        dirs, filename = os.path.split( path )
        dirs = dirs.split(os.sep)

        current = self.brain

        for dir in dirs:
            if not current.has_key( dir ):
                current[dir] = {}
            current = current[dir]

        current[filename] = content

    def read(self, path):
        info( "DictFSEnv: read: %(path)s" % locals() )
        dirs, filename = os.path.split( path )
        dirs = dirs.split(os.sep)

        current = self.brain
        for dir in dirs:
            info( "DictFSEnv: read: traverse to %(dir)s" % locals() )
            if current.has_key(dir):
                current = current[dir]
            else:
                return None

        return current[filename]

    def listdir( self, path ):
        """
        Returns the "files" stored in path

        >>> fs = DictFSEnv({})
        >>> fs.write( "huhu/zonk/blah/a", "a" )
        >>> fs.write( "huhu/zonk/blah/b", "b" )
        >>> fs.write( "huhu/zonk/blah/c", "c" )
        >>> listdir( "huhu/zonk/blah" )
        [ "a", "b", "c" ]
        """
        info( "DictFSEnv: listdir: %(path)s" % locals() )
        return self.lookup(path).keys()

    def isdir( self, path ):
        """
        """
        info( "DictFSEnv: isdir: %(path)s" % locals() )
        d = self.lookup( path )
        if type(d) == type({}):
            return True
        else:
            return False

    def exists( self, path ):
        """
        """
        info( "DictFSEnv: exists: %(path)s" % locals() )
        try:
            d = self.lookup( path )
            return True
        except KeyError:
            return False


class GmailImportContext(GmailContext):
    """
    """
    implements( IImportContext )

    # XXX: hack: the GS code references this but its NOT in the IImportContext
    #            Interface. Doh.
    def getLogger( self, *args, **kw ):
        return logging.getLogger( *args, **kw )

    def __init__( self, name, password, exportid, subject="[GSXML]" ):
        """
        """
        GmailContext.__init__(self, name, password, subject, exportid )

        self.fs = DictFSEnv( {} )

        self.update( )

    def update(self):
        """
        """
        self.messages = self.g_msgquery(self.ga, self.get_subject() )
        for m in self.messages:
            # FIXME: meta/xml sequence fixed??
            meta = m.attachments[0].content
            xml = m.attachments[1].content
            scp = SafeConfigParser()
            scp.readfp( StringIO(meta) )
            self.fs.write( scp.get("info", "filename"), xml )

    def readDataFile( self, filename, subdir=None ):
        """
        See IImportContext.
        """
        info( "GmailImportContext.readDataFile: filename=%s, subdir=%s" % ( filename, subdir ) )

        return self.fs.read( os.path.join( subdir, filename ) )

    def getLastModified( self, path ):
        """
        See IImportContext.
        """
        info( "GmailImportContext.getLastModified: path=%s" % ( path ) )

        if not self.fs.exists( path ):
            return None
        return DateTime(  )

    def isDirectory( self, path ):
        """
        See IImportContext.
        """
        info( "GmailImportContext.isDirectory: path=%s" % ( path ) )

        if not self.fs.exists( path ):
            return None

        return self.fs.isdir( path )

    def listDirectory(self, path, skip=SKIPPED_FILES, skip_suffixes=SKIPPED_SUFFIXES):
        """
        See IImportContext.
        """
        info( "GmailImportContext.listDirectory: path=%s" % ( path ) )
        if path is None:
            path = ''

        if not self.fs.exists( path ) or not self.fs.isdir( path ):
            return None

        names = []
        for name in self.fs.listdir(path):
            if name in skip:
                continue
            if [s for s in skip_suffixes if name.endswith(s)]:
                continue
            names.append(name)

        return names


class SimpleFSExportContext(object):
    """
    see IExportContext
    """
    implements( IExportContext )

    def __init__(self, root):
        """
        """

        #self.dirname = os.path.join( os.environ["HOME"], root )
        self.dirname = root

        if not os.path.isdir( self.dirname ):
            info( "simpleFSExportContext.__init__: make dirs: %s" % self.dirname )
            os.makedirs( self.dirname )


    def writeDataFile( self, filename, text, content_type, subdir=None ):
        """
        """
        info( "simpleFSExportContext.writeDataFile: filename=%s, subdir=%s, content_type=%s" % (
                filename, subdir, content_type ) )

        dir, filename = os.path.split( filename )
        if subdir is not None:
            dir = os.path.join( self.dirname, subdir, dir )
        else:
            dir = os.path.join( self.dirname, dir )
        fullpath = os.path.join( dir, filename )


        if not os.path.isdir( dir ):
            os.makedirs( dir )
            info( "simpleFSExportContext.writeDataFile: make dirs: %s" % dir )

        info( "simpleFSExportContext.writeDataFile: fullpath=%s" % fullpath )
        file( fullpath, "wb" ).write( text )

class SimpleFSImportContext(object):
    """
    see IImportContext
    """
    implements( IImportContext )

    # XXX: hack: the GS code references this but its NOT in the IImportContext
    #            Interface. Doh.
    def getLogger( self, *args, **kw ):
        return logging.getLogger( *args, **kw )

    def __init__( self, root ):
        """
        """
        info( "SimpleFSImportContext.__init__: root=%s" % ( root ) )
        #self.dirname = os.path.join( os.environ["HOME"], root )
        self.dirname = root

    def readDataFile( self, filename, subdir=None ):
        """
        See IImportContext.
        """
        info( "SimpleFSImportContext.readDataFile: filename=%s, subdir=%s" % ( filename, subdir ) )

        if subdir is None:
            full_path = os.path.join( self.dirname, filename )
        else:
            full_path = os.path.join( self.dirname, subdir, filename )

        if not os.path.exists( full_path ):
            return None

        file = open( full_path, 'rb' )
        result = file.read()
        file.close()
        return result


    def getLastModified( self, path ):
        """
        See IImportContext.
        """
        info( "SimpleFSImportContext.getLastModified: path=%s" % ( path ) )
        full_path = os.path.join( self.dirname, path )

        if not os.path.exists( full_path ):
            return None
        return DateTime( os.path.getmtime( full_path ) )

    def isDirectory( self, path ):
        """
        See IImportContext.
        """
        info( "SimpleFSImportContext.isDirectory: path=%s" % ( path ) )
        full_path = os.path.join( self.dirname, path )

        if not os.path.exists( full_path ):
            return None

        return os.path.isdir( full_path )

    def listDirectory(self, path, skip=SKIPPED_FILES, skip_suffixes=SKIPPED_SUFFIXES):
        """
        See IImportContext.
        """
        info( "SimpleFSImportContext.listDirectory: path=%s" % ( path ) )
        if path is None:
            path = ''

        full_path = os.path.join( self.dirname, path )

        if not os.path.exists( full_path ) or not os.path.isdir( full_path ):
            return None

        names = []
        for name in os.listdir(full_path):
            if name in skip:
                continue
            if [s for s in skip_suffixes if name.endswith(s)]:
                continue
            names.append(name)

        return names

# vim: set ft=python ts=4 sw=4 expandtab :
